/***************************************************************************
*   Copyright (C) 2004 by Christoph Thielecke                             *
*   crissi99@gmx.de                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include "importp12certificatedialog.h"
#include <kmessagebox.h>
#include <klocale.h>
#include <kurlrequester.h>
#include <kpassdlg.h>
#include <kcombobox.h>
#include <klineedit.h>
#include <iostream>
#include <qfile.h>
#include <qdir.h>
#include <qtextstream.h>
#include <qurl.h>

ImportP12CertificateDialog::ImportP12CertificateDialog( QTextEdit *LogOutput )
		: ImportP12CertificateDialogBase() {
	this->LogOutput = LogOutput;
	filename = "";
	certHash = "";
	importOk = false;
	doHash = false;
	certName = "";
	racooncertpath = "/etc/racoon/certs/";
	pathToOpenSSL = "/usr/bin/openssl";
	RacoonCertificatePathUrlrequester->setMode( 2 ); // directory
	FilenameUrlrequester->setFilter( "*.p12" );

	typeToggeled( "freeswan" );


	// TMP
	/*
		FilenameUrlrequester->setURL( "/home/crissi/thielecke.p12" );
		//RacoonCertificatePathUrlrequester->setURL( "/etc/racoon/certs/" );
	ImporttypeComboBox->setCurrentText( "racoon" );	
		PrivateKeyPasswordEdit->setText( "12345678" );
		PrivateKeyAgainPasswordEdit->setText( "12345678" );
	*/ 
	//ImportPasswordEdit->setText( "blah99" );

}

ImportP12CertificateDialog::~ImportP12CertificateDialog() {}

void ImportP12CertificateDialog::canAccept() {
	importOk = true;

	filename = FilenameUrlrequester->url();
	importpassword = ImportPasswordEdit->text();
	racooncertpath = RacoonCertificatePathUrlrequester->url();

	if ( filename.isEmpty() ) {
		KMessageBox::sorry( 0, i18n( "Filename can't be empty!" ), i18n( "Empty filename" ) );
		importOk = false;
	}

	if ( filename.right( 3 ) != "p12" ) {
		KMessageBox::sorry( 0, i18n( "Filename isnt ended by p12!" ), i18n( "Wrong filename" ) );
		importOk = false;
	}

	QFile f( filename );
	if ( f.exists() ) {
		if ( f.open( IO_ReadOnly ) ) {
			f.close();
		} else {
			KMessageBox::sorry( 0, i18n( "File doesnt exist!" ), i18n( "No file" ) );
			importOk = false;
		}
	} else {
		KMessageBox::sorry( 0, i18n( "File not readable!" ), i18n( "insufficient rights" ) );
		importOk = false;
	}

	if ( importpassword.isEmpty() ) {
		importOk = false;
		KMessageBox::sorry( 0, i18n( "Password can't be empty!" ), i18n( "Empty password" ) );
	}

	if ( ImporttypeComboBox->currentText() == "freeswan" ) {
		if ( PrivateKeyPasswordEdit->text().isEmpty() || PrivateKeyPasswordEdit->text().length() < 4 ) {
			importOk = false;
			KMessageBox::sorry( 0, i18n( "Private key password field could not be empty or less than 4 characters!" ), i18n( "password empty or too short" ) );
		}

		if ( PrivateKeyAgainPasswordEdit->text().isEmpty() || PrivateKeyAgainPasswordEdit->text().length() < 4 ) {
			importOk = false;
			KMessageBox::sorry( 0, i18n( "Private key password (again) field could not be empty!" ), i18n( "password empty or too short" ) );
		}
		if ( importOk ) {
			if ( PrivateKeyPasswordEdit->text() != PrivateKeyAgainPasswordEdit->text() ) {
				importOk = false;
				KMessageBox::sorry( 0, i18n( "Private key passwords doesnt match!" ), i18n( "passwords not match" ) );
			}
		}
	}

	if ( importOk )
		accept();
}

void ImportP12CertificateDialog::accept() {
	// import now

	// looking for openssl
	if ( pathToOpenSSL.isEmpty() ) {
		QFile openssl( "/usr/sbin/openssl" );
		if ( openssl.exists() )
			pathToOpenSSL = "/usr/sbin/openssl";
		else {
			openssl.setName( "/usr/local/sbin/openssl" );
			if ( openssl.exists() )
				pathToOpenSSL = "/usr/local/sbin/openssl";
			else {
				openssl.setName( "/usr/bin/openssl" );
				if ( openssl.exists() )
					pathToOpenSSL = "/usr/bin/openssl";
				else {
					KMessageBox::error( this, i18n( "Unable to find \"%1\"!" ).arg("openssl") );
					importOk = false;
				}
			}
		}
	}

	QString file = QUrl ( filename ).fileName();
	certName = file.left( file.length() - 4 ); // peter_pan.p12 -> peter_pan

	if ( ImporttypeComboBox->currentText() == "racoon" ) {
		QFile racooncertpathtest( racooncertpath + "/." );

		if ( !racooncertpathtest.exists() ) {
			KMessageBox::error( this, i18n( "racoon certificate path (%1) doesnt exist!" ).arg( racooncertpath ) );
			importOk=false;
		} else {
			QFile testfile ( QString(racooncertpath + "/test" ));
			if ( !testfile.open( IO_WriteOnly ) ) {
				KMessageBox::error( this, i18n( "racoon certificate path (%1) isnt writeable!" ).arg( racooncertpath ) );
				importOk = false;
			}
			else {
				testfile.remove();
			}
		}
	}


	// here we go
	if ( importOk ) {
		/*
		openssl pkcs12 -in cert.p12 -clcerts -out /etc/racoon/certs/mykeys_company.pem -nodes

		*/ 
		// 		 		std::cout << "[std]:" << "settings ok." << std::endl;

		ImportProcess = new QProcess( this );
		ImportProcess->addArgument( pathToOpenSSL );
		ImportProcess->addArgument( "pkcs12" );
		ImportProcess->addArgument( "-in" );
		ImportProcess->addArgument( filename );
		ImportProcess->addArgument( "-clcerts" );
		ImportProcess->addArgument( "-out" );

		if ( ImporttypeComboBox->currentText() == "freeswan" ) {
			ImportProcess->addArgument( "/etc/ipsec.d/certs/mykeys_" + certName + ".pem" ); // warning: static
		} else {
			ImportProcess->addArgument( racooncertpath + "/mykeys_" + certName + ".pem" );
		}

		ImportProcess->addArgument( "-nodes" );

		connect( ImportProcess, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout_import() ) );
		connect( ImportProcess, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr_import() ) );
		connect( ImportProcess, SIGNAL( processExited () ), this, SLOT( doCacert() ) );

		if ( !ImportProcess->start() ) {
			KMessageBox::sorry( this, i18n( "Unable to start process (%1)!" ).arg("openssl") );
		} else {
			// ok
			//std::cout << "[std]:" << " openssl started." << std::endl;
		}
	}
}

void ImportP12CertificateDialog::doCacert() {
	if ( ImportProcess->normalExit () ) {
		//	LogOutput->append("[dbg]: doCacert()");
		disconnect( ImportProcess, SIGNAL( processExited () ), this, SLOT( doCacert() ) );
		/*
		openssl pkcs12 -in cert.p12 -nokeys -cacerts -out /etc/racoon/certs/ca_company.pem -nodes
		*/
		ImportProcess->clearArguments();
		ImportProcess->addArgument( pathToOpenSSL );
		ImportProcess->addArgument( "pkcs12" );
		ImportProcess->addArgument( "-in" );
		ImportProcess->addArgument( filename );
		ImportProcess->addArgument( "-nokeys" );
		ImportProcess->addArgument( "-cacerts" );
		ImportProcess->addArgument( "-out" );

		if ( ImporttypeComboBox->currentText() == "freeswan" )
			ImportProcess->addArgument( "/etc/ipsec.d/cacerts/ca_" + certName + ".pem" ); // warning: static
		else
			ImportProcess->addArgument( racooncertpath + "/ca_" + certName + ".pem" );

		ImportProcess->addArgument( "-nodes" );

		connect( ImportProcess, SIGNAL( processExited () ), this, SLOT( doCreateHash() ) );

		ImportProcess->start();

	}
}

void ImportP12CertificateDialog::doPrivateKey() {
	if ( ImportProcess->normalExit () ) {
		//	LogOutput->append("[dbg]: doPrivateKey()");
		disconnect( ImportProcess, SIGNAL( processExited () ), this, SLOT( doPrivateKey() ) );
		/*
		openssl pkcs12 -in cert.p12 -nocerts -out /etc/racoon/certs/ca_company.pem
		*/
		ImportProcess->clearArguments();
		ImportProcess->addArgument( pathToOpenSSL );
		ImportProcess->addArgument( "pkcs12" );
		ImportProcess->addArgument( "-nocerts" );
		ImportProcess->addArgument( "-in" );
		ImportProcess->addArgument( filename );
		ImportProcess->addArgument( "-out" );
		ImportProcess->addArgument( "/etc/ipsec.d/private/" + certName + ".pem" ); // warning: static

		connect( ImportProcess, SIGNAL( processExited () ), this, SLOT( exit() ) );

		//LogOutput->append( "priv key: we starting..." );
		if ( !ImportProcess->start( ) ) {
			KMessageBox::sorry( this, i18n( "Unable to start process (private key)!" ) );
		} else {}

	}
}

void ImportP12CertificateDialog::doCreateHash() {
	if ( ImportProcess->normalExit () ) {

		disconnect( ImportProcess, SIGNAL( processExited () ), this, SLOT( doCreateHash() ) );

		/*
		openssl x509 -noout -hash < /etc/racoon/certs/ca_company.pem
		*/
		ImportProcess->clearArguments();
		ImportProcess->addArgument( pathToOpenSSL );
		ImportProcess->addArgument( "x509" );
		ImportProcess->addArgument( "-noout" );
		ImportProcess->addArgument( "-hash" );
		ImportProcess->addArgument( "-in" );
		if ( ImporttypeComboBox->currentText() == "freeswan" )
			ImportProcess->addArgument( "/etc/ipsec.d/cacerts/ca_" + certName + ".pem" ); // warning: static
		else
			ImportProcess->addArgument( racooncertpath + "/ca_" + certName + ".pem" );

		connect( ImportProcess, SIGNAL( processExited () ), this, SLOT( doLink() ) );


		//		LogOutput->append( "hash: we starting..." );
		if ( !ImportProcess->start( ) ) {
			KMessageBox::sorry( this, i18n( "Unable to start process (%1)!" ).arg("hash") );
		} else {
			doHash = true;
		}
	}
}

void ImportP12CertificateDialog::doLink() {
	if ( ImportProcess->normalExit () ) {
		disconnect( ImportProcess, SIGNAL( processExited () ), this, SLOT( doLink() ) );
		doHash = false;
		ImportProcess->clearArguments();
		/*
		ln -s /etc/racoon/certs/ca_company.pem /etc/racoon/certs/<hash>.0
		*/
		ImportProcess->addArgument( "ln" );
		ImportProcess->addArgument( "-s" );
		if ( ImporttypeComboBox->currentText() == "freeswan" ) {
			ImportProcess->addArgument( "/etc/ipsec.d/cacerts/ca_" + certName + ".pem" );
			ImportProcess->addArgument( "/etc/ipsec.d/cacerts/" + certHash + ".0" );
		} else {
			ImportProcess->addArgument( racooncertpath + "ca_" + certName + ".pem" );
			ImportProcess->addArgument( racooncertpath + certHash + ".0" );
		}

		if ( ImporttypeComboBox->currentText() == "freeswan" )
			connect( ImportProcess, SIGNAL( processExited () ), this, SLOT( doPrivateKey() ) );
		else
			connect( ImportProcess, SIGNAL( processExited () ), this, SLOT( exit() ) );


		if ( !ImportProcess->start( ) ) {
			KMessageBox::sorry( this, i18n( "Unable to start process (%1)!" ).arg("ln -s") );
		}
	}
}


void ImportP12CertificateDialog::exit() {
	if ( ImportProcess->normalExit() )
		KMessageBox::information( 0, i18n( "Certificate was sucessfully imported." ), i18n( "import successful" ) );
	else
		KMessageBox::sorry( 0, i18n( "Certificate import failed." ), i18n( "import failed" ) );

	QDialog::accept();
}

void ImportP12CertificateDialog::readFromStdout_import() {
	QString line = QString( ImportProcess->readStdout() );

	LogOutput->append( "<font color=\"red\">[std]: " + line + "</font>" );
	if ( doHash ) {
		certHash = line.left( line.length() - 1 );
		doHash = false;
		//		LogOutput->append( "<font color=\"red\">[std]: hash" + line + "</font>" );
	}

	if ( line.contains( "Enter Import Pass" ) > 0 ) {
		LogOutput->append( "<font color=\"green\">[in]: write the password</font>" );
		ImportProcess->writeToStdin( importpassword + "\n" );
	}

	if ( line.contains( "Enter PEM pass phrase" ) > 0 ) {
		LogOutput->append( "<font color=\"green\">[in]: write the private key password</font>" );
		ImportProcess->writeToStdin( PrivateKeyPasswordEdit->text() + "\n" );
	}

}

void ImportP12CertificateDialog::readFromStderr_import() {
	QString line = QString( ImportProcess->readStderr() );

	LogOutput->append( "<font color=\"black\">[err]: " + line + "</font>" );

	if ( line.contains( "invalid password" ) > 0 ) {
		KMessageBox::sorry( 0, i18n( "wrong password?!." ), i18n( "password failed" ) );
	}

	if ( line.contains( "Enter Import Pass" ) > 0 ) {
		LogOutput->append( "<font color=\"green\">[in]: write the password</font>" );
		ImportProcess->writeToStdin( importpassword + "\n" );
	}

	if ( line.contains( "Enter PEM pass phrase" ) > 0 ) {
		LogOutput->append( "<font color=\"green\">[in]: write the private key password</font>" );
		ImportProcess->writeToStdin( PrivateKeyPasswordEdit->text() + "\n" );
	}

	// 	if ( line.contains( "Verifying - Enter PEM pass phrase" ) > 0 ) {
	// 		LogOutput->append( "<font color=\"green\">[in]: write the private key password (again)</font>" );
	// 		ImportProcess->writeToStdin( importpassword + "\n" );
	// 	}


}

void ImportP12CertificateDialog::typeToggeled( const QString & type ) {
	if ( type == "freeswan" ) {
		PrivateKeyPasswordEdit->setEnabled( true );
		PrivateKeyAgainPasswordEdit->setEnabled( true );
		RacoonCertificatePathUrlrequester->setEnabled( false );
	} else {
		PrivateKeyPasswordEdit->setEnabled( false );
		PrivateKeyAgainPasswordEdit->setEnabled( false );
		RacoonCertificatePathUrlrequester->setEnabled( true );
	}
}


#include "importp12certificatedialog.moc"
