/***************************************************************************
*   Copyright (C) 2004 by Christoph Thielecke                             *
*   crissi99@gmx.de                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
// BEGIN Includes
#include "profilewidget.h"
#include "importp12certificatedialog.h"
#include <qlistview.h>
#include <klistview.h>
#include <kcombobox.h>
#include <klineedit.h>
#include <qcheckbox.h>
#include <qtabwidget.h>
#include <kurlrequester.h>
#include <kmessagebox.h>
#include <qlistbox.h>
#include <qlabel.h>
#include <klineeditdlg.h>
#include <klocale.h>
#include <knuminput.h>
#include <kinputdialog.h>
#include <ktextedit.h>
#include "networkinterface.h"
#include "advancedsettingsdialog.h"
#include "newprofiledialog.h"
#include "utils.h"
// END Includes

ProfileWidget::ProfileWidget( QPtrList<VpnAccountData> *AccountList, QTextEdit *LogOutput, int KvpncDebugLevel, VpnAccountData *currentProfile )
{
	this->AccountList = AccountList;
	this->LogOutput = LogOutput;
	this->currentProfile = currentProfile;
	this->KvpncDebugLevel = KvpncDebugLevel;

	// TODO read types from list
	ConnectionTypeComboBox->insertItem( i18n( "Cisco (vpnc)" ), 0 );
	ConnectionTypeComboBox->insertItem( i18n( "IPSec (racoon)" ), 1 );
	ConnectionTypeComboBox->insertItem( i18n( "IPSec (FreeSWAN)" ), 2 );
	ConnectionTypeComboBox->insertItem( i18n( "PPTP" ), 3 );
	ConnectionTypeComboBox->insertItem( i18n( "OpenVPN" ), 4 );
	ConnectionTypeComboBox->insertItem( "-", 5 );

	profileAddedOrDeleted = false;
	NewProfileName="";

	// profile data
	VpnAccountData * it;
	for ( it = AccountList->first(); it; it = AccountList->next() )
	{
		//KMessageBox::information(0,"item found",it->getName());

		ProfileListBox->insertItem( it->getName() );
	}

	connect( ProfileListBox, SIGNAL( highlighted ( QListBoxItem * ) ), this, SLOT( profileChanged( QListBoxItem * ) ) );
	connect( ConnectionTypeComboBox, SIGNAL( activated( const QString& ) ), this, SLOT( connectionTypeChanged( const QString& ) ) );
	connect( AuthTypeComboBox , SIGNAL( activated( const QString& ) ), this, SLOT( authTypeChanged( const QString& ) ) );
	connect( PingCheckBox, SIGNAL( toggled( bool ) ), this, SLOT( pingIpToggled( bool ) ) );


	if ( !AccountList->isEmpty() )
	{
		if ( currentProfile != 0 )
			ProfileListBox->setCurrentItem( ProfileListBox->findItem( currentProfile->getName() ) );
		else
		{
			ProfileListBox->setCurrentItem( 0 );
		}
	}
	else
	{
		AdvancedSettingsPushButton->setEnabled( false );
		RenameSessionPushButton->setEnabled( false );
		DeleteSessionPushButton->setEnabled( false );
		SaveSessionPushButton->setEnabled( false );
		CiscoPptpGroupBox->setEnabled( false );
		FreeswanRacoonGroupBox->setEnabled( false );
		OpenvpnGroupBox->setEnabled( false );
		GeneralGroupbox->setEnabled( false );
		CommandExecutionGroupBox->setEnabled( false );
	}


	NetworkInterface iface;
	QStringList list = iface.getAllNetworkInterfaces();
	NetworkDeviceComboBox->insertItem( "default" );
	for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
	{
		NetworkDeviceComboBox->insertItem( QString( *it ) );
	}
	NetworkDeviceComboBox->setEnabled( true );
	LabelNetworkDevice->setEnabled( true );

	if ( currentProfile != 0 )
	{
		if ( currentProfile->getAuthType() == VpnAccountData::cert )
			AuthTypeComboBox->setCurrentText( i18n( "X.509 certificate" ) );

		if ( currentProfile->getAuthType() == VpnAccountData::psk )
			AuthTypeComboBox->setCurrentText( i18n( "Pre shared key" ) );

		if (currentProfile->getConnectionType() != VpnAccountData::ConnectionType(VpnAccountData::openvpn))
			PSKLineEdit->setText( currentProfile->getPreSharedKey() );
		else
			PreSharedKeyFileURLRequester->setURL(currentProfile->getPreSharedKeyFile());

		if ( currentProfile->getDoPingIP() )
			PingIPLineEdit->setEnabled( true );

		else
			PingIPLineEdit->setEnabled( false );


		RemoteIDLineEdit->setEnabled( false );
		
	}
	if ( currentProfile != 0 )
	{
		connectionTypeChanged( ConnectionTypeComboBox->currentText() );
		authTypeChanged( AuthTypeComboBox->currentText() );
		profileChanged( ProfileListBox->item( ProfileListBox->currentItem() ) );
	}

	connect (ProfileListBox, SIGNAL(doubleClicked ( QListBoxItem *)),this,SLOT(doubleClicked(QListBoxItem *)));
}


ProfileWidget::~ProfileWidget() {}


void ProfileWidget::importCertificate()
{
	ImportP12CertificateDialog dlg( LogOutput );
	//int result =
	dlg.exec();
}

void ProfileWidget::profileChanged( QListBoxItem * )
{

	if (!AccountList->isEmpty() && currentProfile != 0)
	{

		//int item = ProfileListBox->currentItem();
		QString Name = ProfileListBox->currentText();
		if ( KvpncDebugLevel > 0 )
			LogOutput->append( i18n( "New profile: %1" ).arg( Name ) );

		//	KMessageBox::information(0,"item clicked",Name);
		// profile data
		VpnAccountData * it;
		for ( it = AccountList->first(); it; it = AccountList->next() )
		{
			if ( it->getName() == Name )
			{
				currentProfile = it;
				break;
			}
		}

		x509certURLRequester->clear();
		certpathURLRequester->clear();
		UsernameLineEdit->clear();
		GroupPasswordEdit->clear();
		PSKLineEdit->clear();

		CheckGroupPass->setChecked( false );
		CheckUserPass->setChecked( false );
		IDLineEdit->clear();
		UsernameLineEdit->clear();
		UsernameLineEdit->setEnabled( false );
		PasswordEdit->clear();
		PasswordEdit->setEnabled( false );

		CmdConnectTextEdit->setEnabled(false);
		CmdDisconnectTextEdit->setEnabled(false);
		ExcuteCmdAfterConnectCheckBox->setChecked(false);
		ExcuteCmdAfterDisconnectCheckBox->setChecked(false);

		if ( currentProfile == 0 )
		{
			RenameSessionPushButton->setEnabled( false );
			DeleteSessionPushButton->setEnabled( false );
			SaveSessionPushButton->setEnabled( false );
			return;
		}
		else
		{
			AdvancedSettingsPushButton->setEnabled( true );
			DeleteSessionPushButton->setEnabled( true );
			SaveSessionPushButton->setEnabled( true );
		}

		RemoteIDLineEdit->clear();
		RemoteNetMaskSpinBox->setValue(24);

		// 	x509certURLRequester->setEnabled( false );
		// 	certpathURLRequester->setEnabled( false );
		// 	AdvancedSettingsPushButton->setEnabled( false );
		if ( ( currentProfile->getNetworkDevice() ).isEmpty() )
		{
			NetworkDeviceComboBox->setCurrentText( "default" );
			if ( KvpncDebugLevel > 0 )
				LogOutput->append( i18n( "profile \"%1\": no network device definied, using \"default\"." ).arg( currentProfile->getName() ) );
		}
		else
		{
			NetworkDeviceComboBox->setCurrentText( currentProfile->getNetworkDevice() );
			if ( KvpncDebugLevel > 0 )
				LogOutput->append( i18n( "profile \"%1\": network device definied, using \"%2\"." ).arg( currentProfile->getName() ).arg( currentProfile->getNetworkDevice() ) );
		}

		// global
		if ( it->getConnectionType() == VpnAccountData::ConnectionType(VpnAccountData::cisco ))
			ConnectionTypeComboBox->setCurrentItem( 0 );
		else if ( it->getConnectionType() == VpnAccountData::ConnectionType(VpnAccountData::racoon ))
			ConnectionTypeComboBox->setCurrentItem( 1 );
		else if ( it->getConnectionType() == VpnAccountData::ConnectionType(VpnAccountData::freeswan ))
			ConnectionTypeComboBox->setCurrentItem( 2 );
		else if ( it->getConnectionType() == VpnAccountData::ConnectionType(VpnAccountData::pptp ))
			ConnectionTypeComboBox->setCurrentItem( 3 );
		else if ( it->getConnectionType() == VpnAccountData::ConnectionType(VpnAccountData::openvpn ))
			ConnectionTypeComboBox->setCurrentItem( 4 );
		else
			ConnectionTypeComboBox->setCurrentItem( 9 ); // must be last item -> other

		gatewayLineEdit->setText( it->getGateway() );

		PSKLineEdit->setText( currentProfile->getPreSharedKey() );

		if ( it->getConnectionType() == VpnAccountData::cisco )
		{
			CheckGroupPass->setChecked( it->getSaveGroupPassword() );
			CheckUserPass->setChecked ( it->getSaveUserPassword() );
			IDLineEdit->setText( it->getID() );
			UsernameLineEdit->setText ( it->getUserName() );
			GroupPasswordEdit->setText( it->getGroupPassword() );
			PasswordEdit->setText ( it->getUserPassword() );

			if (currentProfile->getConnectionType() != VpnAccountData::ConnectionType(VpnAccountData::openvpn))
				PSKLineEdit->setText( it->getPreSharedKey() );

			CheckGroupPass->setEnabled( true );
			CheckUserPass->setEnabled( true );

			IDLineEdit->setEnabled( true );
			LabelID->setEnabled( true );

			UsernameLineEdit->setEnabled( true );
			LabelUsername->setEnabled ( true);

			GroupPasswordEdit->setEnabled( true );
			LabelGroupPassword->setEnabled( true );

			AdvancedSettingsPushButton->setEnabled( true );

			PasswordEdit->setEnabled( true );
			LabelPassword->setEnabled( true );

			GroupPasswordEdit->setEnabled( true );
			LabelGroupPassword->setEnabled( true );

			x509certURLRequester->setEnabled( false );
			LabelCertificate->setEnabled( false );

			certpathURLRequester->setEnabled( false );
			LabelCertificatePath->setEnabled( false );

			AuthTypeComboBox->setEnabled( false );

			PSKLineEdit->setEnabled( false );
			LabelPsk->setEnabled( false );

			RemoteNetAddrLineEdit->setEnabled( false );
			LabelRemoteNetwork->setEnabled( false );

			RemoteNetMaskSpinBox->setEnabled( false );

			RemoteIDLineEdit->setEnabled( false );

			UseSpecialRemoteIDcheckBox->setEnabled( false );

		} // vpnc

		else if ( it->getConnectionType() == VpnAccountData::ConnectionType(VpnAccountData::racoon) || it->getConnectionType() == VpnAccountData::ConnectionType(VpnAccountData::freeswan ))
		{
			//     KURLRequester* x509certURLRequester;
			//     KComboBox* AuthTypeComboBox;
			//     KPushButton* ImportCertificatePushButton;
			//     KLineEdit* PSKLineEdit;

			QString RemoteNetAddr = it->getRemoteNet().section( '/', 0, 0 );
			int RemoteNetMask = it->getRemoteNet().section( '/', 1, 1 ).toInt();
			RemoteNetAddrLineEdit->setText( RemoteNetAddr );
			RemoteNetMaskSpinBox->setValue( RemoteNetMask );
			if ( it->getAuthType() == VpnAccountData::cert )
				AuthTypeComboBox->setCurrentItem( 0 );
			else
				AuthTypeComboBox->setCurrentItem( 1 );


			x509certURLRequester->setURL( currentProfile->getX509Certificate() );
			certpathURLRequester->setURL( currentProfile->getCertPath() );
			PingCheckBox->setChecked( currentProfile->getDoPingIP() );
			PingIPLineEdit->setText( currentProfile->getPingHostIP() );
			RemoteIDLineEdit->setText( currentProfile->getSpecialRemoteID() );
			UseSpecialRemoteIDcheckBox->setChecked( currentProfile->getUseSpecialRemoteID() );

			FreeswanRacoonGroupBox->setEnabled( true );

			x509certURLRequester->setEnabled( true );
			LabelCertificate->setEnabled( true );

			certpathURLRequester->setEnabled ( true );
			LabelCertificatePath->setEnabled( true );

			AuthTypeComboBox->setEnabled( true );

			PSKLineEdit->setEnabled( true );
			LabelPsk->setEnabled( true );

			RemoteNetAddrLineEdit->setEnabled( true );
			LabelRemoteNetwork->setEnabled( true );

			RemoteNetMaskSpinBox->setEnabled( true );

			UseSpecialRemoteIDcheckBox->setEnabled( true );
			ImportCertificatePushButton->setEnabled( true );

			CheckGroupPass->setEnabled( false );
			CheckUserPass->setEnabled( false );

			IDLineEdit->setEnabled( false );
			LabelID->setEnabled( false );

			UsernameLineEdit->setEnabled( false );
			LabelUsername->setEnabled( false );

			GroupPasswordEdit->setEnabled( false );
			LabelGroupPassword->setEnabled( false );

			//AdvancedSettingsPushButton->setEnabled( false );
			PasswordEdit->setEnabled( false );
			LabelPassword->setEnabled( false );

			if (currentProfile->getConnectionType() == VpnAccountData::racoon)
				AdvancedSettingsPushButton->setEnabled( true );

		} // racoon or freeswan

		else if ( it->getConnectionType() == VpnAccountData::ConnectionType(VpnAccountData::pptp ))
		{
			CheckGroupPass->setChecked( it->getSaveGroupPassword() );
			CheckUserPass->setChecked ( it->getSaveUserPassword() );
			UsernameLineEdit->setText ( it->getUserName() );
			PasswordEdit->setText ( it->getUserPassword() );
			PingCheckBox->setChecked( currentProfile->getDoPingIP() );
			PingIPLineEdit->setText( currentProfile->getPingHostIP() );

			if (!it->getRemoteNet().isEmpty())
			{
				QString RemoteNetAddr = it->getRemoteNet().section( '/', 0, 0 );
				int RemoteNetMask = it->getRemoteNet().section( '/', 1, 1 ).toInt();
				RemoteNetAddrLineEdit->setText( RemoteNetAddr );
				RemoteNetMaskSpinBox->setValue( RemoteNetMask );
			}
			CheckUserPass->setEnabled( true );

			UsernameLineEdit->setEnabled( true );
			LabelUsername->setEnabled( true );

			AdvancedSettingsPushButton->setEnabled( true );

			PasswordEdit->setEnabled( true );
			LabelPassword->setEnabled( true );

			GroupPasswordEdit->setEnabled( false );
			LabelGroupPassword->setEnabled( false );

			IDLineEdit->setEnabled( false );
			LabelID->setEnabled( false );

			CheckGroupPass->setEnabled( false );

			x509certURLRequester->setEnabled( false );
			LabelCertificate->setEnabled( false );

			certpathURLRequester->setEnabled ( false );

			AuthTypeComboBox->setEnabled( false );

			PSKLineEdit->setEnabled( false );
			LabelPsk->setEnabled( false );

			RemoteNetAddrLineEdit->setEnabled( true );
			LabelRemoteNetwork->setEnabled( true );

			RemoteNetMaskSpinBox->setEnabled(true );

			UseSpecialRemoteIDcheckBox->setEnabled( false );
			ImportCertificatePushButton->setEnabled( false );
			NetworkDeviceComboBox->setEnabled( false );

			FreeswanRacoonGroupBox->setEnabled( false );

		} // pptp

		else if ( it->getConnectionType() == VpnAccountData::ConnectionType(VpnAccountData::openvpn ))
		{
			//     KURLRequester* x509certURLRequester;
			//     KComboBox* AuthTypeComboBox;
			//     KPushButton* ImportCertificatePushButton;
			//     KLineEdit* PSKLineEdit;

			FreeswanRacoonGroupBox->setEnabled( true );

			if ( it->getAuthType() == VpnAccountData::cert )
			{
				AuthTypeComboBox->setCurrentItem( 0 );
				ImportCertificatePushButton->setEnabled( true );
				x509certURLRequester->setEnabled( true );
				LabelCertificate->setEnabled( true );
				certpathURLRequester->setEnabled ( true );
				LabelCertificatePath->setEnabled( true );
			}
			else
			{
				AuthTypeComboBox->setCurrentItem( 1 );
				PSKLineEdit->setEnabled( false );
				LabelPsk->setEnabled( false );
			}

			x509certURLRequester->setURL( currentProfile->getX509Certificate() );
			certpathURLRequester->setURL( currentProfile->getCertPath() );
			PingCheckBox->setChecked( currentProfile->getDoPingIP() );
			PingIPLineEdit->setText( currentProfile->getPingHostIP() );

			LocalVirtualIpEdit->setText( currentProfile->getLocalVirtualIP());
			RemoteVirtualIpLineEdit->setText( currentProfile->getRemoteVirtualIP());
			useTcpCheckBox->setChecked ( currentProfile->getUseUdp() );

			AuthTypeComboBox->setEnabled( true );

			CheckGroupPass->setEnabled( false );
			CheckUserPass->setEnabled( false );

			IDLineEdit->setEnabled( false );
			LabelID->setEnabled( false );

			UsernameLineEdit->setEnabled( false );
			LabelUsername->setEnabled( false );

			GroupPasswordEdit->setEnabled( false );
			LabelGroupPassword->setEnabled( false );

			//AdvancedSettingsPushButton->setEnabled( false );
			PasswordEdit->setEnabled( false );
			LabelPassword->setEnabled( false );

			useTcpCheckBox->setEnabled ( true );
			// 		AdvancedSettingsPushButton->setEnabled( true );


		} // openvpn

		ExcuteCmdAfterConnectCheckBox->setChecked(currentProfile->getExecuteCmdAfterConnect());
		ExcuteCmdAfterDisconnectCheckBox->setChecked(currentProfile->getExecuteCmdAfterDisconnect());
		CmdConnectTextEdit->setText(currentProfile->getCommandAfterConnect());
		CmdDisconnectTextEdit->setText (currentProfile->getCommandAfterDisconnect());
		CmdConnectTextEdit->setEnabled(currentProfile->getExecuteCmdAfterConnect());
		CmdDisconnectTextEdit->setEnabled(currentProfile->getExecuteCmdAfterDisconnect());
		
		connectionTypeChanged( ConnectionTypeComboBox->currentText() );
		authTypeChanged( AuthTypeComboBox->currentText() );
		x509certURLRequester->setFilter( "*.pem" );
		certpathURLRequester->setMode( KFile::Directory );
	}
}

void ProfileWidget::connectionTypeChanged( const QString& )
{

	DeleteSessionPushButton->setEnabled ( TRUE );
	NewSessionPushButton->setEnabled ( TRUE );
	SaveSessionPushButton->setEnabled ( TRUE );

	// First we should set a type
	// 	parent->buttonCancel->setEnabled( FALSE );
	// 	parent->buttonOk->setEnabled( FALSE );
	AdvancedSettingsPushButton->setEnabled( false );

	int item = ConnectionTypeComboBox->currentItem();

	if ( item == VpnAccountData::ConnectionType( VpnAccountData::cisco ) )
	{
		if ( KvpncDebugLevel > 0 )
			LogOutput->append( "new type: cisco" );

		CheckGroupPass->setEnabled( true );
		CheckUserPass->setEnabled( true );

		IDLineEdit->setEnabled( true );
		LabelID->setEnabled( true );

		UsernameLineEdit->setEnabled( true );
		LabelUsername->setEnabled( true );

		GroupPasswordEdit->setEnabled( true );
		LabelGroupPassword->setEnabled( true );

		PasswordEdit->setEnabled( true );
		LabelPassword->setEnabled( true );

		AdvancedSettingsPushButton->setEnabled( true );

		// 		x509certURLRequester->setEnabled( false );
		// 		certpathURLRequester->setEnabled( false );
		NetworkDeviceComboBox->setEnabled( true );
		LabelNetworkDevice->setEnabled( true );

		AuthTypeComboBox->setEnabled( false );
		LabelAuthType->setEnabled( false );

		// 		PSKLineEdit->setEnabled( false );
		// 		buttonCancel->setEnabled( TRUE );
		// 		buttonOk->setEnabled( TRUE );
		AdvancedSettingsPushButton->setEnabled( true );

		RemoteNetAddrLineEdit->setEnabled( false );
		LabelRemoteNetwork->setEnabled( false );

		RemoteNetMaskSpinBox->setEnabled( false );

		privkeypathURLRequester->setEnabled( false );
		LabelPrivateKeyPath->setEnabled( false );

		PrivkeyPasswordEdit->setEnabled( false );
		LabelPrivateKeyPassword->setEnabled( false );

		ImportCertificatePushButton->setEnabled( false );

		OpenvpnGroupBox->setEnabled( false );
	} // vpnc

	else if ( item == VpnAccountData::ConnectionType(VpnAccountData::racoon) || item == VpnAccountData::ConnectionType(VpnAccountData::freeswan ))
	{

		if ( item == VpnAccountData::ConnectionType(VpnAccountData::racoon ))
		{
			if ( KvpncDebugLevel > 0 )
				LogOutput->append( i18n( "New type: %1" ).arg("racoon") );

			privkeypathURLRequester->setEnabled( false );
			LabelPrivateKeyPath->setEnabled( false );

			PrivkeyPasswordEdit->setEnabled( false );
			LabelPrivateKeyPassword->setEnabled( false );

			AdvancedSettingsPushButton->setEnabled( true );
		}
		else
		{
			if ( KvpncDebugLevel > 0 )
				LogOutput->append( i18n( "New type: %1" ).arg("freeswan") );
			privkeypathURLRequester->setEnabled( true );
			LabelPrivateKeyPath->setEnabled( true );

			PrivkeyPasswordEdit->setEnabled( true );
			LabelPassword->setEnabled( true );

			if (currentProfile != 0)
			{
				privkeypathURLRequester->setURL( currentProfile->getPrivateKey() );
				PrivkeyPasswordEdit->setText( currentProfile->getPrivateKeyPass() );
			}
			AdvancedSettingsPushButton->setEnabled( false );
		}
		//     KURLRequester* x509certURLRequester;
		//     KComboBox* AuthTypeComboBox;
		//     KPushButton* ImportCertificatePushButton;
		//     KLineEdit* PSKLineEdit;

		// 		x509certURLRequester->setEnabled( true );
		// 		certpathURLRequester->setEnabled( true );
		//ImportCertificatePushButton->setEnabled( true );
		NetworkDeviceComboBox->setEnabled ( true );
		LabelNetworkDevice->setEnabled( true );

		AuthTypeComboBox->setEnabled( true );
		LabelAuthType->setEnabled( true );

		// 		PSKLineEdit->setEnabled( true );
		RemoteNetAddrLineEdit->setEnabled( true );
		LabelRemoteNetwork->setEnabled( true );

		RemoteNetMaskSpinBox->setEnabled( true );

		PasswordEdit->setEnabled( false );
		LabelPassword->setEnabled( false );

		CheckGroupPass->setEnabled( false );
		CheckUserPass->setEnabled( false );

		IDLineEdit->setEnabled( false );
		LabelID->setEnabled( false );

		GroupPasswordEdit->setEnabled( false );
		LabelGroupPassword->setEnabled( false );

		PasswordEdit->setEnabled( false );
		LabelPassword->setEnabled( false );

		UsernameLineEdit->setEnabled( false );
		LabelUsername->setEnabled( false );

		OpenvpnGroupBox->setEnabled( false );

		// 		buttonCancel->setEnabled( TRUE );
		// 		buttonOk->setEnabled( TRUE );
		if ( UseSpecialRemoteIDcheckBox->isChecked() )
			RemoteIDLineEdit->setEnabled( true );
		else
			RemoteIDLineEdit->setEnabled( false );
	} // racoon/freeswan

	else if ( item == VpnAccountData::ConnectionType( VpnAccountData::pptp ) )
	{
		if ( KvpncDebugLevel > 0 )
			LogOutput->append( "new type: pptp" );

		CheckUserPass->setEnabled( true );

		UsernameLineEdit->setEnabled( true );
		LabelUsername->setEnabled( true );

		PasswordEdit->setEnabled( true );
		LabelPassword->setEnabled( true );

		AdvancedSettingsPushButton->setEnabled( true );

		OpenvpnGroupBox->setEnabled( false );
	} // pptp
	else if ( item == VpnAccountData::ConnectionType(VpnAccountData::openvpn ))
	{

		if ( KvpncDebugLevel > 0 )
			LogOutput->append( i18n( "new type: %1" ).arg("openvpn") );

		privkeypathURLRequester->setEnabled( false );
		LabelPrivateKeyPath->setEnabled( false );

		PrivkeyPasswordEdit->setEnabled( false );
		LabelPrivateKeyPassword->setEnabled( false );

		AdvancedSettingsPushButton->setEnabled( false );

		//     KURLRequester* x509certURLRequester;
		//     KComboBox* AuthTypeComboBox;
		//     KPushButton* ImportCertificatePushButton;
		//     KLineEdit* PSKLineEdit;

		// 		x509certURLRequester->setEnabled( true );
		// 		certpathURLRequester->setEnabled( true );
		//ImportCertificatePushButton->setEnabled( true );
		NetworkDeviceComboBox->setEnabled ( false );
		LabelNetworkDevice->setEnabled( false );

		AuthTypeComboBox->setEnabled( true );
		LabelAuthType->setEnabled( true );

		// 		PSKLineEdit->setEnabled( true );
		RemoteNetAddrLineEdit->setEnabled( false );
		LabelRemoteNetwork->setEnabled( false );

		RemoteNetMaskSpinBox->setEnabled( false );

		CheckGroupPass->setEnabled( false );
		CheckUserPass->setEnabled( false );

		IDLineEdit->setEnabled( false );
		LabelID->setEnabled( false );

		GroupPasswordEdit->setEnabled( false );
		LabelGroupPassword->setEnabled( false );

		PasswordEdit->setEnabled( false );
		LabelPassword->setEnabled( false );

		UsernameLineEdit->setEnabled( false );
		LabelUsername->setEnabled( false );

		// 		buttonCancel->setEnabled( TRUE );
		// 		buttonOk->setEnabled( TRUE );

		OpenvpnGroupBox->setEnabled( true );
	} // openvpn

	// other
	else if ( item == VpnAccountData::other )
	{
		if ( KvpncDebugLevel > 0 )
			LogOutput->append( i18n( "new type: none specified" ) );
		// 		x509certURLRequester->setEnabled( false );
		// 		AuthTypeComboBox->setEnabled( false );
		// 		PSKLineEdit->setEnabled( false );
		//
		// 		CheckGroupPass->setEnabled( false );
		// 		CheckUserPass->setEnabled( false );
		// 		IDLineEdit->setEnabled( false );
		// 		UsernameLineEdit->setEnabled( false );
		// 		GroupPasswordEdit->setEnabled( false );
		// 		AdvancedSettingsPushButton->setEnabled( false );
		// 		buttonCancel->setEnabled( FALSE );
		// 		buttonOk->setEnabled( FALSE );
	}
	else
	{
		LogOutput->append( i18n( "unknown type" ) );
		OpenvpnGroupBox->setEnabled( false );
	}
	if ( currentProfile != 0 )
	{
		authTypeChanged( "" );
	}
}

void ProfileWidget::authTypeChanged( const QString& )
{

	// racoon or freeswan
	if ( ConnectionTypeComboBox->currentItem() == 1 || ConnectionTypeComboBox->currentItem() == 2 )
	{


		// cert
		if ( AuthTypeComboBox->currentItem() == 0 )
		{
			// certificate
			PSKLineEdit->setEnabled( false );
			LabelPsk->setEnabled( false );

			x509certURLRequester->setEnabled( true );
			LabelCertificate->setEnabled( true );

			certpathURLRequester->setEnabled( true );
			LabelCertificatePath->setEnabled( true );

			ImportCertificatePushButton->setEnabled( true );


		}

		// psk
		if ( AuthTypeComboBox->currentItem() == 1 )
		{
			// psk
			PSKLineEdit->setEnabled( true );
			LabelPsk->setEnabled( true );

			x509certURLRequester->setEnabled( false );
			LabelCertificate->setEnabled( false );

			certpathURLRequester->setEnabled( false );
			LabelCertificatePath->setEnabled( false );

			ImportCertificatePushButton->setEnabled( false );
		}
	}
	else if (currentProfile->getConnectionType() == VpnAccountData::ConnectionType(VpnAccountData::openvpn))
	{
		// cert
		if ( AuthTypeComboBox->currentItem() == 0 )
		{
			// certificate

			x509certURLRequester->setEnabled( true );
			LabelCertificate->setEnabled( true );

			//certpathURLRequester->setEnabled( true );
			//LabelCertificatePath->setEnabled( true );

			ImportCertificatePushButton->setEnabled( true );


			PreSharedKeyFileURLRequester->setEnabled( false );
			LabelPrivateKeyPath->setEnabled( false );
		}

		// psk
		if ( AuthTypeComboBox->currentItem() == 1 )
		{
			// psk
			PreSharedKeyFileURLRequester->setEnabled( true );
			LabelPrivateKeyPath->setEnabled( true );

			x509certURLRequester->setEnabled( false );
			LabelCertificate->setEnabled( false );

			certpathURLRequester->setEnabled( false );
			LabelCertificatePath->setEnabled( false );

			ImportCertificatePushButton->setEnabled( false );
		}

	}
}

void ProfileWidget::newSessionClicked()
{

	// call new profile dialog...
	NewProfileDialog newdlg( this, i18n("New profile"),AccountList, LogOutput );
	newdlg.exec();
	if ( newdlg.newProfileCreated )
	{
		QString Name = newdlg.profileData->getName();
		VpnAccountData * it;
		for ( it = AccountList->first(); it; it = AccountList->next() )
		{
			if ( it->getName() == Name )
			{
				ProfileListBox->insertItem( Name, 0 );
				ProfileListBox->setCurrentItem ( 0 );
				currentProfile = it;
				break;
			}
		}

		AdvancedSettingsPushButton->setEnabled( TRUE );
		RenameSessionPushButton->setEnabled( TRUE );
		DeleteSessionPushButton->setEnabled( TRUE );
		SaveSessionPushButton->setEnabled ( TRUE );

		CiscoPptpGroupBox->setEnabled( true );
		FreeswanRacoonGroupBox->setEnabled( true );
		OpenvpnGroupBox->setEnabled( true );
		GeneralGroupbox->setEnabled( true );
		CommandExecutionGroupBox->setEnabled( true );

		KMessageBox::information ( this, i18n( "Profile \"%1\" added.\nNow set the right type,\n fill in the fields\nand press \"Save\" to complete the profile." ).arg( Name ), i18n( "Complete profile" ) );
		//slotStatusMsg ( i18n( "Profile \"%1\" added." ).arg( Name ), ID_FLASH_MSG );
		LogOutput->append ( i18n( "Profile \"%1\" added." ).arg( Name ) );
		saveSessionClicked();
		profileAddedOrDeleted=true;
		NewProfileName=Name;
	}
	else
	{
		LogOutput->append ( i18n( "Creating new profile canceled." ) );
	}

}


void ProfileWidget::saveSessionClicked()
{
	if ( currentProfile== 0 )
		return;
	//QString itemString=ConnectionTypeComboBox->currentText();
	int item = ConnectionTypeComboBox->currentItem();
	// cisco
	if ( item == VpnAccountData::cisco )
		currentProfile->setConnectionType( VpnAccountData::cisco );
	else if ( item == VpnAccountData::racoon || item == VpnAccountData::freeswan )
	{
		currentProfile->setSpecialRemoteID( RemoteIDLineEdit->text() );
		currentProfile->setUseSpecialRemoteID( UseSpecialRemoteIDcheckBox->isChecked() );

		if ( item == VpnAccountData::racoon )
			currentProfile->setConnectionType( VpnAccountData::racoon );
		else
		{
			currentProfile->setConnectionType( VpnAccountData::freeswan );
			currentProfile->setPrivateKey( privkeypathURLRequester->url() );
			currentProfile->setPrivateKeyPass( PrivkeyPasswordEdit->text() );
		}

		bool validAddr = true;
		if ( RemoteNetAddrLineEdit->text().contains( '.' ) != 3 )
		{
			KMessageBox::error ( this, i18n( "No IP address entered!" ), i18n( "No IP address" ) );
			validAddr = false;
		}
		else
		{
			QString addr = RemoteNetAddrLineEdit->text();
			int part0 = addr.section( '.', 0, 0 ).toInt();
			int part1 = addr.section( '.', 1, 1 ).toInt();
			int part2 = addr.section( '.', 2, 2 ).toInt();
			int part3 = addr.section( '.', 3, 3 ).toInt();

			/*
						LogOutput->append ("part0: "+QString().setNum(part0));
						LogOutput->append ("part1: "+QString().setNum(part1));
						LogOutput->append ("part2: "+QString().setNum(part2));
						LogOutput->append ("part3: "+QString().setNum(part3));
			*/
			if ( ( part0 < 1 || part0 > 254 ) || ( part1 < 0 || part1 > 254 ) || ( part2 < 0 || part2 > 254 ) || ( part3 < 0 || part3 > 254 ) )
			{
				KMessageBox::error ( this, i18n( "IP address (remote net) is invalid!" ), i18n( "Invalid IP address" ) );
				validAddr = false;
			}
			if ( validAddr == true )
			{
				currentProfile->setRemoteNet( addr + "/" + QString().setNum( RemoteNetMaskSpinBox->value() ) );
			}

		}

		if ( AuthTypeComboBox->currentItem() == 0 )
			currentProfile->setAuthType( VpnAccountData::cert );
		else
			currentProfile->setAuthType( VpnAccountData::psk );

		currentProfile->setX509Certificate( x509certURLRequester->url() );
		currentProfile->setCertPath( certpathURLRequester->url() );

		currentProfile->setPreSharedKey( PSKLineEdit->text());


	}

	else if ( item == VpnAccountData::pptp )
	{
		currentProfile->setConnectionType( VpnAccountData::pptp );


		bool validAddr = true;
		if (!RemoteNetAddrLineEdit->text().isEmpty())
		{
			if ( RemoteNetAddrLineEdit->text().contains( '.' ) != 3 )
			{
				//KMessageBox::error ( this, i18n( "No IP address entered!" ), i18n( "No IP address" ) );
				validAddr = false;
			}
			else
			{
				QString addr = RemoteNetAddrLineEdit->text();
				int part0 = addr.section( '.', 0, 0 ).toInt();
				int part1 = addr.section( '.', 1, 1 ).toInt();
				int part2 = addr.section( '.', 2, 2 ).toInt();
				int part3 = addr.section( '.', 3, 3 ).toInt();

				/*
							LogOutput->append ("part0: "+QString().setNum(part0));
							LogOutput->append ("part1: "+QString().setNum(part1));
							LogOutput->append ("part2: "+QString().setNum(part2));
							LogOutput->append ("part3: "+QString().setNum(part3));
				*/

				if ( ( part0 < 1 || part0 > 254 ) || ( part1 < 0 || part1 > 254 ) || ( part2 < 0 || part2 > 254 ) || ( part3 < 0 || part3 > 254 ) )
				{
					KMessageBox::error ( this, i18n( "Invalid values in IP address (remote net)!" ), i18n( "Invalid values in IP address" ) );
					validAddr = false;
				}


				if ( validAddr == true )
				{
					currentProfile->setRemoteNet( addr + "/" + QString().setNum( RemoteNetMaskSpinBox->value() ) );
				}

			}
		}
		else
		{
			currentProfile->setRemoteNet("");
		}

	}

	else if ( item == VpnAccountData::ConnectionType(VpnAccountData::openvpn) )
	{
		currentProfile->setConnectionType( VpnAccountData::openvpn );

		QString LocalVirtualIp = LocalVirtualIpEdit->text();
		if (Utils().isValidIPv4Adress(LocalVirtualIp) == false)
			KMessageBox::error ( this, i18n( "No valid IP address entered!" ), i18n( "No valid IP address" ) );
		else
		{
			currentProfile->setLocalVirtualIP( LocalVirtualIp);
		}

		QString RemoteVirtualIp = RemoteVirtualIpLineEdit->text();
		if (Utils().isValidIPv4Adress(RemoteVirtualIp) == false)
			KMessageBox::error ( this, i18n( "No valid IP address entered!" ), i18n( "No valid IP address" ) );
		else
		{
			currentProfile->setRemoteVirtualIP( RemoteVirtualIp);
		}

		// tmp
		AuthTypeComboBox->setCurrentItem(1); // psk only (yet)

		if ( AuthTypeComboBox->currentItem() == 0 )
		{
			currentProfile->setAuthType( VpnAccountData::cert );
			currentProfile->setX509Certificate( x509certURLRequester->url() );
			currentProfile->setCertPath( certpathURLRequester->url() );

		}
		else
		{
			currentProfile->setAuthType( VpnAccountData::psk );
			currentProfile->setPreSharedKeyFile(PreSharedKeyFileURLRequester->url());
		}

		currentProfile->setUseUdp( useTcpCheckBox->isChecked());

	} // openvpn

	else
		currentProfile->setConnectionType( VpnAccountData::other );

	if ( PingCheckBox->isChecked() && !PingIPLineEdit->text().isEmpty() )
	{
		// do check for vailid IP
		if (!Utils().isValidIPv4Adress(PingIPLineEdit->text()))
			KMessageBox::error ( this, i18n( "No valid IP address entered!" ), i18n( "No valid IP address" ) );
		else
		{
			currentProfile->setPingHostIP( PingIPLineEdit->text() );
			currentProfile->setDoPingIP( PingCheckBox->isChecked());
		}
	}
	else
	{
		if ( PingCheckBox->isChecked() && PingIPLineEdit->text() == QString( "" ) )
			KMessageBox::error ( this, i18n( "No IP address (ping host) entered!" ), i18n( "No IP address" ) );
		else
			currentProfile->setPingHostIP( "" );
	}


	currentProfile->setSaveGroupPassword( CheckGroupPass->isChecked() );
	currentProfile->setSaveUserPassword( CheckUserPass->isChecked () );
	currentProfile->setID( IDLineEdit->text() );
	currentProfile->setUserName( UsernameLineEdit->text () );

	if (CheckGroupPass->isChecked())
		currentProfile->setGroupPassword( GroupPasswordEdit->text( ) );
	else
		currentProfile->setGroupPassword( "" );

	if (CheckUserPass->isChecked())
		currentProfile->setUserPassword( PasswordEdit->text ( ) );
	else
		currentProfile->setUserPassword( "" );

	currentProfile->setGateway( gatewayLineEdit->text() );
	currentProfile->setPreSharedKey( PSKLineEdit->text() );
	currentProfile->setX509Certificate( x509certURLRequester->url() );
	currentProfile->setCertPath( certpathURLRequester->url() );
	currentProfile->setNetworkDevice( NetworkDeviceComboBox->currentText() );

	currentProfile->setDoPingIP( PingCheckBox->isChecked() );

	if ( AuthTypeComboBox->currentItem() == 0 )
		currentProfile->setAuthType( VpnAccountData::cert );
	if ( AuthTypeComboBox->currentItem() == 1 )
		currentProfile->setAuthType( VpnAccountData::psk );

	currentProfile->setExecuteCmdAfterConnect( (ExcuteCmdAfterConnectCheckBox->isChecked()));
	currentProfile->setExecuteCmdAfterDisconnect( (ExcuteCmdAfterDisconnectCheckBox->isChecked()));

	currentProfile->setCommandAfterConnect(CmdConnectTextEdit->text());
	currentProfile->setCommandAfterDisconnect(CmdDisconnectTextEdit->text());

	LogOutput->append( i18n( "Profile \"%1\" saved." ).arg( currentProfile->getName() ) );
}


void ProfileWidget::deleteSessionClicked()
{
	QString Name = ProfileListBox->currentText();
	if ( !ProfileListBox->currentText().isEmpty() )
	{
		int result = KMessageBox::questionYesNo ( this, i18n( "Really delete \"%1\"?" ).arg( Name ), i18n( "Delete?" ) );
		//LogOutput->append( "Result: "+ QString().setNum(result) ) ;
		if ( result == 3) // Yes
		{


			VpnAccountData * it;
			for ( it = AccountList->first(); it; it = AccountList->next() )
			{
				if ( ProfileListBox->currentText().isEmpty() )
				{
					// 					this->deleteSessionAction->setEnabled ( FALSE );
					//			this->advancedSessionAction->setEnabled ( FALSE );
					// 					this->saveSessionAction->setEnabled ( FALSE );
				}
				else
				{
					if ( it->getName() == Name )
					{
						currentProfile=0L;
						AccountList->remove( it );
						ProfileListBox->removeItem( ProfileListBox->currentItem() );
						break;
					}
				}
			}


			if ( !AccountList->isEmpty() )
			{
				AccountList->first();
				ProfileListBox->setCurrentItem( 0 );
				// 				buttonCancel->setEnabled( true );
				// 				buttonOk->setEnabled( true );
			}
			else
			{
				this->DeleteSessionPushButton->setEnabled ( FALSE );
				this->RenameSessionPushButton->setEnabled ( FALSE );
				this->SaveSessionPushButton->setEnabled ( FALSE );
				AdvancedSettingsPushButton->setEnabled( false );
				CiscoPptpGroupBox->setEnabled( false );
				FreeswanRacoonGroupBox->setEnabled( false );
				OpenvpnGroupBox->setEnabled( false );
				GeneralGroupbox->setEnabled( false );
				CommandExecutionGroupBox->setEnabled( false );
			}

			//		slotStatusMsg ( i18n( "Profile \"%1\" deleted." ).arg( Name ), ID_FLASH_MSG );
			LogOutput->append ( i18n( "Profile \"%1\" deleted." ).arg( Name ) );
			saveSessionClicked();
			profileAddedOrDeleted=true;
			//saveOptions(); // TODO save, currently after closing dialog
			//			sessionToggled();
			//if ( ProfileListBox->currentText().isEmpty() )
			//	ProfileListBox->setEnabled ( FALSE );
		}
	}

}

void ProfileWidget::slotCancelClicked()
{
	cancelClicked = true;
}

void ProfileWidget::advancedSettingsClicked()
{
	AdvancedSettingsDialog d1(currentProfile,currentProfile->getConnectionType());

	d1.LocalPortCheckbox->setEnabled( false );
	d1.IkeGroupCheckbox->setEnabled( false );
	d1.PerfectForwardSecrecyCombobox->setEnabled( false );
	d1.SingleDesCheckbox->setEnabled( false );
	d1.NtDomainNameCheckbox->setEnabled( false );
	d1.LocalPortSpinbox->setEnabled( false );
	d1.ApplicationVersionLineedit->setEnabled( false );
	d1.NtDomainNameLineedit->setEnabled( false );
	d1.PerfectForwardSecurityCheckbox->setEnabled( false );
	d1.IkeGroupCombobox->setEnabled( false );
	d1.ApplicationVersionCheckbox->setEnabled( false );
	d1.UseGlobalIpsecSecretCheckbox->setEnabled( false );
	d1.RefuseEapCheckbox->setEnabled( false );
	d1.requireMppeCheckbox->setEnabled( false );
	d1.Refuse128BitEncryptionCheckbox->setEnabled(false );
	d1.Refuse40BitEncryptionCheckbox->setEnabled( false );
	d1.DnsServerCheckbox->setEnabled( false );
	d1.DnsServerLineedit->setEnabled( false );
	d1.AllowStatefulModeCheckbox->setEnabled( false );
	d1.GetDnsFromPeerCheckbox->setEnabled( false );
	d1.IkeGroupCheckbox->setEnabled( false );
	d1.UdpPortSpinbox->setEnabled( false );
	d1.UseUdpCheckbox->setEnabled( false );
	d1.UdpPortCheckbox->setEnabled( false );
	d1.SetDefaultRouteCheckbox->setEnabled( false );
	d1.ReplaceDefaultRouteCheckbox->setEnabled( false );
	d1.MtuCheckbox->setEnabled(false);
	d1.MtuSpinbox->setEnabled(false);


	d1.UseGlobalIpsecSecretCheckbox->setChecked ( currentProfile->getUseGlobalIpsecSecret() );
	d1.SingleDesCheckbox->setChecked( currentProfile->getUseSpecialRemoteID() );
	d1.IkeGroupCheckbox->setChecked ( currentProfile->getUseIkeGroup() );

	d1.PerfectForwardSecurityCheckbox->setChecked( currentProfile->getUsePerfectForwardSecrety() );
	d1.NtDomainNameCheckbox->setEnabled ( currentProfile->getUseNtDomainName() );
	d1.ApplicationVersionCheckbox->setEnabled ( currentProfile->getUseApplicationVersion() );
	d1.UseGlobalIpsecSecretCheckbox->setEnabled ( currentProfile->getUseGlobalIpsecSecret() );

	d1.LocalPortSpinbox->setValue( currentProfile->getLocalPort() );
	if (!currentProfile->getIkeGroup().isEmpty())
		d1.IkeGroupCombobox->setCurrentText( currentProfile->getIkeGroup() );
	d1.PerfectForwardSecrecyCombobox->setCurrentText( currentProfile->getPerfectForwardSecrety() );
	d1.NtDomainNameLineedit->setText( currentProfile->getNtDomainName() );
	d1.ApplicationVersionLineedit->setText( currentProfile->getApplicationVersion() );

	if (currentProfile->connectiontype == VpnAccountData::pptp)
	{
		d1.RefuseEapCheckbox->setChecked( currentProfile->getRefuseEap() );
		d1.requireMppeCheckbox->setChecked( currentProfile->getRequireMppe() );
		d1.Refuse128BitEncryptionCheckbox->setChecked( currentProfile->getRefuse128BitEncryption() );
		d1.Refuse40BitEncryptionCheckbox->setChecked( currentProfile->getRefuse40BitEncryption() );
		d1.DnsServerCheckbox->setChecked( currentProfile->getUseDnsServer() );
		d1.DnsServerLineedit->setText( currentProfile->getDnsServer() );
		d1.MtuCheckbox->setChecked(currentProfile->getUseMtu());
		d1.MtuSpinbox->setValue(currentProfile->getMtu());
	}
	d1.NtDomainNameCheckbox->setChecked( currentProfile->getUseNtDomainName() );
	d1.NtDomainNameLineedit->setText( currentProfile->getNtDomainName() );
	d1.AllowStatefulModeCheckbox->setChecked( currentProfile->getAllowStatefulMode() );
	d1.GetDnsFromPeerCheckbox->setChecked( currentProfile->getUseGetDnsFromPeer() );
	d1.AdvancedSettingsCheckbox->setChecked ( currentProfile->getUseAdvancedSettings());
	d1.UdpPortCheckbox->setChecked(currentProfile->getUseUdpPort());
	d1.UseUdpCheckbox->setChecked(currentProfile->getUseUdp());
	d1.UdpPortSpinbox->setValue(currentProfile->getUdpPort());
	d1.SetDefaultRouteCheckbox->setChecked(currentProfile->getSetDefaultRoute());
	d1.ReplaceDefaultRouteCheckbox->setChecked(currentProfile->getReplaceDefaultRoute());

	if ( currentProfile->getUseAdvancedSettings() )
	{
		if (currentProfile->connectiontype == VpnAccountData::cisco)
		{
			d1.LocalPortCheckbox->setEnabled ( true );
			d1.PerfectForwardSecurityCheckbox->setEnabled ( true );
			d1.SingleDesCheckbox->setEnabled ( true );
			d1.NtDomainNameCheckbox->setEnabled ( true );
			d1.ApplicationVersionCheckbox->setEnabled ( true );
			d1.UseGlobalIpsecSecretCheckbox->setEnabled ( true );
			d1.UseUdpCheckbox->setEnabled(true);
			// 			if (currentProfile->getUseUdp())
			d1.UdpPortCheckbox->setEnabled(true);
			// 			else
			// 			  d1.UdpPortCheckbox->setEnabled(false);
		}

		if (currentProfile->connectiontype == VpnAccountData::racoon)
		{
			d1.UseUdpCheckbox->setEnabled(true);
			if (currentProfile->getUseUdp())
				d1.UdpPortCheckbox->setEnabled(true);
			else
				d1.UdpPortCheckbox->setEnabled(false);
		}

		else if (currentProfile->connectiontype == VpnAccountData::pptp)
		{
			d1.NtDomainNameCheckbox->setEnabled ( true );
			d1.RefuseEapCheckbox->setEnabled( true );
			d1.requireMppeCheckbox->setEnabled( true );
			d1.Refuse128BitEncryptionCheckbox->setEnabled( true );
			d1.Refuse40BitEncryptionCheckbox->setEnabled( true );
			d1.DnsServerCheckbox->setEnabled( true );
			d1.DnsServerLineedit->setEnabled( true );
			d1.NtDomainNameCheckbox->setEnabled( true );
			//d1.NtDomainNameLineedit->setEnabled( true );
			d1.AllowStatefulModeCheckbox->setEnabled( true );
			d1.GetDnsFromPeerCheckbox->setEnabled( true );
			//d1.UdpPortCheckbox->setEnabled(true);
			//d1.UseUdpCheckbox->setEnabled(true);
			d1.SetDefaultRouteCheckbox->setEnabled( true );
			if (currentProfile->getSetDefaultRoute())
				d1.ReplaceDefaultRouteCheckbox->setEnabled( true );

			d1.IkeGroupCheckbox->setEnabled( true );
			if (!currentProfile->getIkeGroup().isEmpty())
				d1.IkeGroupCombobox->setCurrentText(currentProfile->getIkeGroup());
			
			d1.MtuCheckbox->setEnabled(true);
		}


		d1.IkeGroupCheckbox->setChecked ( currentProfile->getUseIkeGroup() );

		d1.SingleDesCheckbox->setChecked( currentProfile->getUseSingleDes() );

		d1.IkeGroupCheckbox->setChecked ( currentProfile->getUseIkeGroup() );

		d1.UseGlobalIpsecSecretCheckbox->setChecked ( currentProfile->getUseGlobalIpsecSecret() );

		d1.PerfectForwardSecurityCheckbox->setChecked( currentProfile->getUsePerfectForwardSecrety() );

		if ( currentProfile->getUseNtDomainName() )
		{
			d1.NtDomainNameLineedit->setEnabled ( true );
			d1.NtDomainNameCheckbox->setChecked( true );
		}

		if ( currentProfile->getUseApplicationVersion() )
		{
			d1.ApplicationVersionLineedit->setEnabled ( true );
			d1.ApplicationVersionCheckbox->setChecked( true );
		}

		if ( currentProfile->getUseLocalPort() )
		{
			d1.LocalPortSpinbox->setEnabled ( true );
			d1.LocalPortCheckbox->setChecked( true );
		}

		d1.IkeGroupCombobox->setEnabled (  currentProfile->getUseIkeGroup() );

		d1.UseUdpCheckbox->setChecked( currentProfile->getUseUdp() );

		d1.UdpPortCheckbox->setChecked( currentProfile->getUseUdpPort() );

		d1.UdpPortSpinbox->setValue(currentProfile->getUdpPort());

		d1.UseNoBsdCompCheckbox->setChecked(currentProfile->getUseNoBsdComp());

		d1.UseNoDeflateCheckbox->setChecked(currentProfile->getUseNoDeflate());
	}

	d1.exec();

	saveSessionClicked();

}

void ProfileWidget::pingIpToggled( bool )
{
	if ( PingCheckBox->isChecked() )
	{
		PingIPLineEdit->setEnabled( true );
	}
	else
	{
		PingIPLineEdit->setEnabled( false );
	}
}

void ProfileWidget::useSpecialRemoteIDClicked()
{
	if ( UseSpecialRemoteIDcheckBox->isChecked() )
		RemoteIDLineEdit->setEnabled( true );
	else
		RemoteIDLineEdit->setEnabled( false );
}

void ProfileWidget::doubleClicked(QListBoxItem *)
{
	advancedSettingsClicked();

}

void ProfileWidget::renameSessionClicked()
{
	bool ok=false;
	QString oldName= currentProfile->getName();
	QString newName = KInputDialog::getText ( i18n("New name"),i18n("Enter new name for profile:"), oldName, &ok,this);

	if (ok)
	{
		currentProfile->setName(newName);
		int currIndex = ProfileListBox->currentItem();
		ProfileListBox->removeItem(currIndex);
		ProfileListBox->insertItem(newName, currIndex);
		ProfileListBox->setCurrentItem(currIndex);

		saveSessionClicked();

		LogOutput->append( i18n( "Profile \"%1\" renamed to \"%2\"." ).arg(oldName).arg( currentProfile->getName() ) );
	}

}

void ProfileWidget::executeCmdAfterDisconnectClicked()
{
	if (ExcuteCmdAfterDisconnectCheckBox->isChecked())
		CmdDisconnectTextEdit->setEnabled( TRUE );
	else
		CmdDisconnectTextEdit->setEnabled( FALSE );
}


void ProfileWidget::executeCmdAfterConnectClicked()
{
	if (ExcuteCmdAfterConnectCheckBox->isChecked())
		CmdConnectTextEdit->setEnabled( TRUE );
	else
		CmdConnectTextEdit->setEnabled( FALSE );
}


#include "profilewidget.moc"
